/**
 * Title:        mike.bid.com
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Charles E Hughes
 * @version 1.0
 */

// A client to use the bid server

import java.util.Vector;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.NotBoundException;
import java.rmi.Naming;
import java.rmi.server.UnicastRemoteObject;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.MalformedURLException;

public class BidClientImpl extends UnicastRemoteObject implements BidClient {

    String name = "Anonymous";
    BidServer server;
    BidFrame guiWindow=null;
    int uid;
    Vector bidList=new Vector();

    public BidClientImpl(String n, BidServer s) throws java.rmi.RemoteException {
      name = n;
      server = s;
      uid=server.register(this);
      // set up and display GUI
      guiWindow=new BidFrame(this,n);
      guiWindow.show();
    }

    public void updateBidList(Vector bidList) throws RemoteException {
      // sent by server on every change to bid list
      this.bidList=bidList;
      if (guiWindow!=null) guiWindow.setOfferings(bidList);
    }

    public void purchase(BidClient seller, String item, int price) throws RemoteException {
      // I bought it
      guiWindow.statusBox.setText("Bid Accepted: you bought "+item+" from "+seller.whoAreYou()+" for $"+price);
    }

    public void sale(BidClient buyer, String item, int price) throws RemoteException {
      // I sold it
      guiWindow.statusBox.setText("Sale complete: "+item+" was sold to "+buyer.whoAreYou()+" for $"+price);
    }

    public String whoAreYou() throws RemoteException {
      // used by buyer and seller to exchange names and addresses
      return name;
    }

    public void friend(BidClient buddy) throws RemoteException {
      // just here to test clinet to client communication
      System.err.println(buddy.whoAreYou() + " is a friend of mine");
    }

    public int getUID() throws RemoteException {
      return(uid);
    }

    public static void main(String[] args) {
      if (args.length != 2) {
        System.err.println("Usage: BidClientImpl <url> name");
        System.exit(1);
      }

      if (System.getSecurityManager() == null) {
        System.setSecurityManager(new RMISecurityManager());
      }
      Remote r = null;
      try {
        r = Naming.lookup(args[0]);
      } catch (RemoteException ex) {
        System.err.println("Couldn't contact registry.");
        System.err.println("Are you sure you're running rmiregistry?");
        System.exit(1);
      } catch (NotBoundException ex) {
        System.err.println("There is no object bound to " + args[0]);
        System.err.println("Are you sure you ran the server?");
        System.exit(1);
      } catch (MalformedURLException ex) {
        System.err.println("The string " + args[0] + " is not a valid RMI URL");
        System.err.println("Make sure you use a properly-formatted rmi:// URL");
        System.exit(1);
      }
      try {
        if (r instanceof BidServer) {
	  BidServer server = (BidServer) r;
          BidClientImpl bidder = new BidClientImpl(args[1], server);
          System.out.println("BidClientImpl created");
        } else {
          System.err.println("The object from RMI must be a BidServer.");
        }
      } catch (RemoteException ex) {
        System.err.println("Problems: " + ex.getMessage());
        ex.printStackTrace();
      }
   }
}
