/**
 * Title:        mike.bid.com
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Charles E Hughes
 * @version 1.0
 */

// the GUI stuff for the client

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.rmi.RemoteException;

public class BidFrame extends JFrame {
  JPanel contentPane;
  JLabel bidListLabel = new JLabel();
  JButton offerButton = new JButton();
  JButton acceptButton = new JButton();
  JButton bidButton = new JButton();
  JTextField itemField = new JTextField();
  JLabel itemLabel = new JLabel();
  JTextField priceField = new JTextField();
  JLabel priceLabel = new JLabel();
  JComboBox offeringsList = new JComboBox();
  JTextField statusBox = new JTextField();
  JTextPane titleText = new JTextPane();
  JLabel userLabel = new JLabel();

  BidClientImpl client;  // link back to the client
  boolean blockComboUpdate=false;

  /**Construct the frame*/
  public BidFrame(BidClient client, String userStr) {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    } catch(Exception e) {
      e.printStackTrace();
    }
    userLabel.setText("User: "+userStr);
    this.client=(BidClientImpl)client;
    setOfferings(this.client.bidList);
  }
  /**Component initialization*/
  private void jbInit() throws Exception  {
    //setIconImage(Toolkit.getDefaultToolkit().createImage(BidClient.class.getResource("[Your Icon]")));
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(null);
    this.setSize(new Dimension(400, 300));
    this.setTitle("Bid Client");
    bidListLabel.setHorizontalAlignment(SwingConstants.LEFT);
    bidListLabel.setText("Bid List");
    bidListLabel.setBounds(new Rectangle(204, 56, 88, 20));
    offerButton.setMaximumSize(new Dimension(64, 24));
    offerButton.setMinimumSize(new Dimension(64, 24));
    offerButton.setPreferredSize(new Dimension(64, 24));
    offerButton.setText("Offer");
    offerButton.setBounds(new Rectangle(11, 61, 81, 23));
    offerButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        offerButton_actionPerformed(e);
      }
    });
    acceptButton.setMaximumSize(new Dimension(64, 24));
    acceptButton.setMinimumSize(new Dimension(64, 24));
    acceptButton.setPreferredSize(new Dimension(64, 24));
    acceptButton.setText("Accept");
    acceptButton.setBounds(new Rectangle(11, 89, 84, 24));
    acceptButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        acceptButton_actionPerformed(e);
      }
    });
    bidButton.setPreferredSize(new Dimension(64, 24));
    bidButton.setMinimumSize(new Dimension(64, 24));
    bidButton.setText("Bid");
    bidButton.setMaximumSize(new Dimension(64, 24));
    bidButton.setBounds(new Rectangle(12, 119, 82, 23));
    bidButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        bidButton_actionPerformed(e);
      }
    });
    itemField.setBounds(new Rectangle(12, 170, 153, 25));
    itemLabel.setText("Item Name");
    itemLabel.setBounds(new Rectangle(13, 150, 152, 18));
    priceField.setBounds(new Rectangle(12, 225, 151, 26));
    priceLabel.setText("Bid Amount");
    priceLabel.setBounds(new Rectangle(13, 205, 150, 17));
    offeringsList.setBounds(new Rectangle(204, 78, 189, 28));
    offeringsList.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        offeringsList_actionPerformed(e);
      }
    });
    statusBox.setText("Welcome to mike.bid.com!");
    statusBox.setBounds(new Rectangle(8, 267, 384, 26));
    titleText.setFont(new java.awt.Font("Dialog", 1, 30));
    titleText.setText("mike.bid.com");
    titleText.setBackground(Color.lightGray);
    titleText.setBounds(new Rectangle(12, 6, 233, 47));
    userLabel.setText("User: ");
    userLabel.setBounds(new Rectangle(254, 2, 143, 24));
    contentPane.add(itemLabel, null);
    contentPane.add(bidButton, null);
    contentPane.add(acceptButton, null);
    contentPane.add(offerButton, null);
    contentPane.add(offeringsList, null);
    contentPane.add(bidListLabel, null);
    contentPane.add(statusBox, null);
    contentPane.add(titleText, null);
    contentPane.add(userLabel, null);
    contentPane.add(itemField, null);
    contentPane.add(priceLabel, null);
    contentPane.add(priceField, null);

  }
  /**Overridden so we can exit when window is closed*/
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      try {
        client.server.unregister(client.getUID());
      } catch (RemoteException re) {
        re.printStackTrace();
      }
      System.exit(0);
    }
  }

  void offerButton_actionPerformed(ActionEvent e) {
    boolean okay;
    try {
      okay=client.server.offer(client.uid, itemField.getText(), Integer.parseInt(priceField.getText()));
      if (!okay) statusBox.setText("Can't offer, over limit or duplicate item");
      else statusBox.setText("Offer accepted, "+itemField.getText()+" is now up for sale.");
    } catch (RemoteException re) {
      statusBox.setText("Network error!!!");
    }
  }

  void acceptButton_actionPerformed(ActionEvent e) {
    boolean okay;
    try {
      okay=client.server.accept(client.uid, itemField.getText());
      if (!okay) statusBox.setText("Can't accept, you are not the owner or no bids were placed");
    } catch (RemoteException re) {
      statusBox.setText("Network error!!!");
    }
  }

  void bidButton_actionPerformed(ActionEvent e) {
    boolean okay;
    try {
      okay=client.server.bid(client.uid, itemField.getText(), Integer.parseInt(priceField.getText()));
      if (!okay) statusBox.setText("Can't place bid, Your bid was too low or item doesn't exist");
      else statusBox.setText("You are now the high bidder for "+itemField.getText());
    } catch (RemoteException re) {
      statusBox.setText("Network error!!!");
    }
  }


  void setOfferings(Vector bidList) {
    int i=0;
    offeringsList.removeAllItems();
    Bid bidNode;
    String item;
    String price;

    blockComboUpdate=true;
    for (i=0; i<bidList.size(); i++) {
      bidNode=(Bid)bidList.elementAt(i);
      item=bidNode.getName();
      price=new String("$");
      price+=String.valueOf(bidNode.getPrice());
      offeringsList.addItem(item + " at " + price);
      //offeringsList.revalidate();
    }
    offeringsList.revalidate();
    blockComboUpdate=false;
  }

  void offeringsList_actionPerformed(ActionEvent e) {
    int idx;
    Bid bidNode;
    idx=offeringsList.getSelectedIndex();
    if (idx>=0 && !blockComboUpdate) {
      bidNode=(Bid)client.bidList.elementAt(idx);
      itemField.setText(bidNode.getName());
      priceField.setText(String.valueOf(bidNode.getPrice()));
    }
  }
}